/**************************************************************************
* libXicq - Originally written by tnc (xtrophy@it.dk)                     *
*'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''*
* packets.cpp                                                             *
* Create packets                                                          *
**************************************************************************/


#include "packets.h"


extern struct NET_INFORMATION net_information;
extern struct USER_CONFIGURATION user_config;

struct ICQ_PACKET packet;

int c_sequence = 1;         // Sequence number
int s_sequence = 1;         // Search sequence


/*
** 
** Creates a header
*/
void packets_header(struct ICQ_PACKET *packet)
{
    unsigned short int version;
    unsigned short int command;
    unsigned short int sequence;
    unsigned long  int uin;
    unsigned char *tmp; 

    version  = (unsigned short int)ICQ_VER;
    command  = (unsigned short int)packet->command;
    sequence = (unsigned short int)packet->sequence;
    uin      =  (unsigned long int)user_config.uin;

    tmp = packet->data;

    memcpy(tmp, &version, 2);   tmp += 2;
    memcpy(tmp, &command, 2);   tmp += 2;
    memcpy(tmp, &sequence, 2);  tmp += 2;
    memcpy(tmp, &uin, 4);       tmp += 4;

    packet->length = tmp - packet->data;
}


/*
** 
** Creates an ack packet
*/
struct ICQ_PACKET packets_ack(int sequence)
{
    packet.command = C_ACK;
    packet.sequence = sequence;
    packets_header(&packet);

    return packet;
}


/*
** 
** Creates a message ack packet
*/
struct ICQ_PACKET packets_msgAck(int sequence)
{
    packet.command = C_ACK_MSG;
    packet.sequence = sequence;
    packets_header(&packet);

    return packet;
}


/*
** 
** Creates a keep alive packet
*/
struct ICQ_PACKET packets_keepAlive()
{
    packet.command = C_KEEP_ALIVE;
    packet.sequence = c_sequence++;
    packets_header(&packet);

    return packet;
}


/*
** 
** Creates a login packet
*/
struct ICQ_PACKET packets_login()
{
    unsigned long  int port;
    unsigned short int pwdlen;
    char pwd[NAMESIZE];
    unsigned long int   X1;
    unsigned long int   ip;
    unsigned char       X2;
    unsigned long int   status;
    unsigned long int   X3;
    unsigned short int  lseq;
    unsigned long int   X4;
    unsigned long int   X5; 
    unsigned char *tmp;

    packet.command = C_LOGIN;
    packet.sequence = c_sequence++;
    packets_header(&packet);
    
    port     = net_information.clientPort;
    pwdlen   = strlen(user_config.password) + 1;
    strcpy(pwd, user_config.password);
    X1	= 120;
    ip	= inet_addr(net_information.clientAddr);
    X2	= 04;
    status	= 0;
    X3	= 2;
    lseq	= 1;
    X4	= 0;
    X5	= 7864328;

    tmp = packet.data + packet.length;

    memcpy(tmp, &port, 4);      tmp += 4;
    memcpy(tmp, &pwdlen, 2);    tmp += 2;
    memcpy(tmp, pwd, pwdlen);   tmp += pwdlen;
    memcpy(tmp, &X1, 4);        tmp += 4;
    memcpy(tmp, &ip, 4);     	tmp += 4;
    memcpy(tmp, &X2, 1);      	tmp += 1;
    memcpy(tmp, &status, 4);    tmp += 4;
    memcpy(tmp, &X3, 4);       	tmp += 4;
    memcpy(tmp, &lseq, 2);     	tmp += 2;
    memcpy(tmp, &X4, 4);       	tmp += 4;
    memcpy(tmp, &X5, 4);       	tmp += 4;

    packet.length = tmp - packet.data;

    return packet;
}


/*
** 
** This is sent during login, don't know the exact purpose
*/
struct ICQ_PACKET packets_login_1()
{
    packet.command = C_LOGIN_1;
    packet.sequence = c_sequence++;
    packets_header(&packet);

    return packet;
}


/*
** 
** Creates a contactList
*/
struct ICQ_PACKET packets_contactList()
{
    int i;
    unsigned char *tmp;
    unsigned short int length;
    struct CONTACT contact;

    packet.command = C_CONTACT_LIST;
    packet.sequence = c_sequence++;
    packets_header(&packet);

    tmp = packet.data + packet.length;

    length = contact_length();
    memcpy(tmp, &length, 1);    tmp += 1;

    for (i = 0; i < contact_length(); i++)
    {
        contact = contact_returnItemNum(i);
        memcpy(tmp, &contact.uin, 4);   tmp += 4;
    }

    packet.length = tmp - packet.data;

    return packet;
}


/*
** 
** Creates a message to be sent through the server
*/
struct ICQ_PACKET packets_messageS(unsigned long int receiver, char *message)
{
    unsigned char *tmp;
    unsigned short int type;
    unsigned short int msgLength;

    packet.command = C_SEND_MESSAGE;
    packet.sequence = c_sequence++;
    packets_header(&packet);

    tmp = packet.data + packet.length;

    type = 1; // Normal message
    msgLength = strlen(message) + 1;

    memcpy(tmp, &receiver, 4);          tmp += 4;
    memcpy(tmp, &type, 2);              tmp += 2;
    memcpy(tmp, &msgLength, 2);         tmp += 2;
    memcpy(tmp, message, msgLength);    tmp += msgLength;

    packet.length = tmp - packet.data;
    
    return packet;
}


/*
**
** Creates a URL to be sent through the server.
*/
struct ICQ_PACKET packets_urlS(unsigned long int receiver, char *url, char *desc)
{
    unsigned char *tmp;
    unsigned short int type;
    unsigned char message[BUFSIZE];
    unsigned short int msgLength;
    int i;

    packet.command = C_SEND_MESSAGE;
    packet.sequence = c_sequence++;
    packets_header(&packet);

    tmp = packet.data + packet.length;

    type = 4; // URL

    for(i=0;i<=BUFSIZE;i++) message[i]=0;
    strcpy((char *)message, desc);
    message[strlen((char *)message)] = 0xFE;
    message[strlen((char *)message) + 1] = '\0';

    strcat((char *)message, url);

    msgLength = strlen((char *)message) + 1;

    memcpy(tmp, &receiver, 4);          tmp += 4;
    memcpy(tmp, &type, 2);              tmp += 2;
    memcpy(tmp, &msgLength, 2);         tmp += 2;
    memcpy(tmp, message, msgLength);    tmp += msgLength;

    packet.length = tmp - packet.data;

    return packet;
}

  
/*
** 
** Change your online status
*/
struct ICQ_PACKET packets_statusChange(unsigned long int status)
{
    unsigned char *tmp; 

    packet.command = C_STATUS_CHANGE;
    packet.sequence = c_sequence++;
    packets_header(&packet);

    tmp = packet.data + packet.length;

    memcpy(tmp, &status, 4);            tmp += 4;

    packet.length = tmp - packet.data;

    return packet;
}


/*
** 
** Search for info on a user (UIN search)
*/
struct ICQ_PACKET packets_uinSearch(unsigned long int uin)
{
    unsigned char *tmp;

    packet.command = C_SEARCH_UIN;
    packet.sequence = c_sequence++;
    packets_header(&packet);

    tmp = packet.data + packet.length;

    memcpy(tmp, &s_sequence, 2);   tmp += 2;
    memcpy(tmp, &uin, 4);               tmp += 4;

    packet.length = tmp - packet.data;

    s_sequence++;

    return packet;
}


/*
** 
** Send a text code to the server. Used for logoff etc
*/
struct ICQ_PACKET packets_sendTextCode(char *textCode, unsigned short int X1)
{
    unsigned char *tmp;
    unsigned short int  length;

    packet.command = C_SEND_TEXT_CODE;
    packet.sequence = c_sequence++;
    packets_header(&packet);

    tmp = packet.data + packet.length;

    length = strlen(textCode) + 1;

    memcpy(tmp, &length, 2);            tmp += 2;
    memcpy(tmp, &textCode, length);     tmp += length;
    memcpy(tmp, &X1, 2);                tmp += 2;

    packet.length = tmp - packet.data;
    
    return packet;
}


